package com.xjrsoft.module.job;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xjrsoft.common.utils.WeChatUtil;
import com.xjrsoft.module.attendance.entity.AttendanceRuleDetails;
import com.xjrsoft.module.attendance.entity.AttendanceUserRelation;
import com.xjrsoft.module.attendance.mapper.AttendanceUserRelationMapper;
import com.xjrsoft.module.attendance.service.IAttendanceRuleCategoryService;
import com.xjrsoft.module.concat.service.IXjrUserService;
import com.xjrsoft.module.organization.dto.WeChatSendMessageDto;
import com.xjrsoft.module.organization.service.IWeChatService;
import com.xjrsoft.module.system.entity.WechatMessageLog;
import com.xjrsoft.module.system.service.IWechatMessageLogService;
import com.xjrsoft.module.teacher.entity.XjrUser;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 考勤预警通知，提醒教师和学生是否该上课等
 * @author dzx
 * @date 2024年5月21日
 */
@Component
@Slf4j
public class AttenDanceWarnNoticeTask {
    @Autowired
    private IAttendanceRuleCategoryService categoryService;

    @Autowired
    private IXjrUserService userService;

    @Autowired
    private AttendanceUserRelationMapper relationMapper;

    @Autowired
    private IWeChatService weChatService;

    @Autowired
    private WeChatUtil weChatUtil;

    @Autowired
    private IWechatMessageLogService wechatMessageLogService;

    @Scheduled(cron = "0 */10 * * * ?")
    public void RefreshConnectionPool() {
        String active = SpringUtil.getActiveProfile();
        if(!"prod".equals(active)){
            log.info("非正式环境，无法执行数据推送");
            return;
        }
        //获取当前时间
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String timeStr = now.format(formatter);

        //查询今天所有的考勤规则
        List<AttendanceRuleDetails> ruleDetailsList = categoryService.getTodayRules();
        Map<Long, AttendanceRuleDetails> ruleDetailsMap = new HashMap<>();
        for (AttendanceRuleDetails ruleDetail : ruleDetailsList) {
            ruleDetailsMap.put(ruleDetail.getAttendanceRuleCategoryId(), ruleDetail);
        }

        //查询出所有人的考勤规则
        List<AttendanceUserRelation> relationList = relationMapper.selectList(
                new QueryWrapper<AttendanceUserRelation>().lambda()
                        .eq(AttendanceUserRelation::getDeleteMark, 0)
        );
        Map<Long, AttendanceRuleDetails> allTodyRule = new HashMap<>();
        for (AttendanceUserRelation userRelation : relationList) {
            allTodyRule.put(userRelation.getUserId(), ruleDetailsMap.get(userRelation.getAttendanceRuleCategoryId()));
        }

        //判断所有当前时间下，哪几条规则需要进行提醒
        Map<Long, Integer> ruleDetailsJudgeMap = judgeExecuteNotice(now, ruleDetailsList);

        //查询

        //根据规则中的用户id查询出所有绑定了微信的学生和教师
        if(allTodyRule.isEmpty()){
            log.info("未能查询出规则，不进行推送");
            return;
        }
        String wechatTemplate = weChatUtil.getAttenDanceWarnTemplate();
        WechatMessageLog messageLog = new WechatMessageLog();
        messageLog.setTemplateId(wechatTemplate);
//        messageLog.setSendTime(recentlyTime);

        List<XjrUser> userList = getUserList(allTodyRule.keySet());
        for (XjrUser xjrUser : userList) {
            AttendanceRuleDetails ruleDetails = allTodyRule.get(xjrUser.getId());
            Integer status = ruleDetailsJudgeMap.get(ruleDetails.getId());
            if(status == 0){
                continue;
            }

            WeChatSendMessageDto weChatSendMessageDto = new WeChatSendMessageDto();
            weChatSendMessageDto.setUserId(xjrUser.getOpenId());
            weChatSendMessageDto.setTemplateId(wechatTemplate);
            weChatSendMessageDto.setMsgId(xjrUser.getId().toString());
            JSONObject paramJson = new JSONObject();

            JSONObject thing8 = new JSONObject();
            thing8.put("value", xjrUser.getName());
            paramJson.put("thing8", thing8);

            JSONObject time9 = new JSONObject();
            time9.put("value", timeStr);
            paramJson.put("time9", time9);

            JSONObject const12 = new JSONObject();
            if(status == 1){
                const12.put("value", "进校");
            }else{
                const12.put("value", "出校");
            }
            paramJson.put("const12", const12);

            weChatSendMessageDto.setContent(paramJson);
            weChatService.sendTemplateMessage(weChatSendMessageDto);

        }

        messageLog.setCreateDate(new Date());
        messageLog.setContent("消息推送人数：" + userList.size());
        wechatMessageLogService.save(messageLog);

    }

    /**
     * 判断所有规则，哪些需要发送
     * @param judgeTime
     * @param ruleDetailsList
     * @return 0：不需要提醒，-1：待打卡，1：未打卡
     */
    Map<Long, Integer> judgeExecuteNotice(LocalDateTime judgeTime, List<AttendanceRuleDetails> ruleDetailsList){
        Map<Long, Integer> result = new HashMap<>();
        for (AttendanceRuleDetails ruleDetails : ruleDetailsList) {
            if(ruleDetails.getIsAllowInOutSchool() != null && ruleDetails.getIsAllowInOutSchool() != 1
                    && ruleDetails.getIsAttendance() != null && ruleDetails.getIsAttendance() == 1){
                if(ruleDetails.getAmStartTime() != null){
                    LocalDateTime amStartTime = judgeTime.with(ruleDetails.getAmStartTime());
                    long amBetween = ChronoUnit.MINUTES.between(judgeTime, amStartTime);
                    if(amBetween == ruleDetails.getAgoMinutes()){
                        result.put(ruleDetails.getId(), - 1);
                    }else if(amBetween == ruleDetails.getOverMinutes()){
                        result.put(ruleDetails.getId(), - 1);
                    }
                }
                if(ruleDetails.getPmStartTime() != null){
                    LocalDateTime pmStartTime = judgeTime.with(ruleDetails.getPmStartTime());
                    long pmBetween = ChronoUnit.MINUTES.between(judgeTime, pmStartTime);
                    if(pmBetween == ruleDetails.getAgoMinutes()){
                        result.put(ruleDetails.getId(), - 1);
                    }else if(pmBetween == ruleDetails.getOverMinutes()){
                        result.put(ruleDetails.getId(), - 1);
                    }
                }

                if(ruleDetails.getEveningStartTime() != null){
                    LocalDateTime eveningStartTime = judgeTime.with(ruleDetails.getEveningStartTime());
                    long eveningBetween = ChronoUnit.MINUTES.between(judgeTime, eveningStartTime);
                    if(eveningBetween == ruleDetails.getAgoMinutes()){
                        result.put(ruleDetails.getId(), - 1);
                    }else if(eveningBetween == ruleDetails.getOverMinutes()){
                        result.put(ruleDetails.getId(), - 1);
                    }
                }


                if(!result.containsKey(ruleDetails.getId())){
                    result.put(ruleDetails.getId(), 0);
                }
            }else{
                result.put(ruleDetails.getId(), 0);
            }

        }
        return result;
    }
    /**
     * 查询出所有用户信息（openId不为空的）
     * @param userIds 需要插叙的userid
     * @return 用户信息
     */
    List<XjrUser> getUserList(Set<Long> userIds){
        return userService.list(
            new QueryWrapper<XjrUser>().lambda()
            .in(XjrUser::getId, userIds)
            .isNotNull(XjrUser::getOpenId)
        );
    }



}
